package org.snarfed.android.headphonebuttons;

import android.app.Application;
import android.content.pm.ApplicationInfo;
import android.os.Build;
import android.util.Log;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Implements an application object for the plug-in.
 * <p>
 * This application is non-essential for the plug-in's operation; it simply enables debugging options globally for the app.
 */
public final class PluginApplication extends Application
{
    @Override
    public void onCreate()
    {
        super.onCreate();

        if ((getApplicationInfo().flags & ApplicationInfo.FLAG_DEBUGGABLE) != 0)
        {
            if (Constants.IS_LOGGABLE)
            {
                Log.v(Constants.LOG_TAG, "Application is debuggable.  Enabling additional debug logging"); //$NON-NLS-1$
            }

            /*
             * If using the Fragment compatibility library, enable debug logging here
             */
            // FragmentManager.enableDebugLogging(true);
            // LoaderManager.enableDebugLogging(true);

            if (Build.VERSION.SDK_INT >= 9)
            {
                try
                {
                    final Class<?> strictModeClass = Class.forName("android.os.StrictMode"); //$NON-NLS-1$
                    final Method enableDefaultsMethod = strictModeClass.getMethod("enableDefaults"); //$NON-NLS-1$
                    enableDefaultsMethod.invoke(strictModeClass);
                }
                catch (final ClassNotFoundException e)
                {
                    throw new RuntimeException(e);
                }
                catch (final SecurityException e)
                {
                    throw new RuntimeException(e);
                }
                catch (final NoSuchMethodException e)
                {
                    throw new RuntimeException(e);
                }
                catch (final IllegalArgumentException e)
                {
                    throw new RuntimeException(e);
                }
                catch (final IllegalAccessException e)
                {
                    throw new RuntimeException(e);
                }
                catch (final InvocationTargetException e)
                {
                    throw new RuntimeException(e);
                }
            }
        }
    }
}
