package org.snarfed.android.headphonebuttons.receiver;

import java.io.IOException;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.ComponentName;
import android.content.Intent;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.provider.MediaStore;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.widget.Toast;

import org.snarfed.android.headphonebuttons.Constants;
import org.snarfed.android.headphonebuttons.bundle.BundleScrubber;
import org.snarfed.android.headphonebuttons.bundle.PluginBundleManager;
import org.snarfed.android.headphonebuttons.ui.EditActivity;

/**
 * This is the "fire" BroadcastReceiver for a Locale Plug-in setting.
 */
public final class FireReceiver extends BroadcastReceiver
{

    /**
     * @param context {@inheritDoc}.
     * @param intent the incoming {@link com.twofortyfouram.locale.Intent#ACTION_FIRE_SETTING} Intent. This should contain the
     *            {@link com.twofortyfouram.locale.Intent#EXTRA_BUNDLE} that was saved by {@link EditActivity} and later broadcast
     *            by Locale.
     */
    @Override
    public void onReceive(final Context context, final Intent intent)
    {
        /*
         * Always be sure to be strict on input parameters! A malicious third-party app could always send an empty or otherwise
         * malformed Intent. And since Locale applies settings in the background, the plug-in definitely shouldn't crash in the
         * background.
         */

        /*
         * Locale guarantees that the Intent action will be ACTION_FIRE_SETTING
         */
        if (!com.twofortyfouram.locale.Intent.ACTION_FIRE_SETTING.equals(intent.getAction()))
        {
            if (Constants.IS_LOGGABLE)
            {
                Log.e(Constants.LOG_TAG, String.format("Received unexpected Intent action %s %s", intent.getAction(), intent.getParcelableExtra(Intent.EXTRA_KEY_EVENT))); //$NON-NLS-1$
            }
            return;
        }

        /*
         * A hack to prevent a private serializable classloader attack
         */
        BundleScrubber.scrub(intent);
        BundleScrubber.scrub(intent.getBundleExtra(com.twofortyfouram.locale.Intent.EXTRA_BUNDLE));

        final Bundle bundle = intent.getBundleExtra(com.twofortyfouram.locale.Intent.EXTRA_BUNDLE);

        /*
         * Final verification of the plug-in Bundle before firing the setting.
         */
        if (PluginBundleManager.isBundleValid(bundle))
        {
            int playingPos = bundle.getInt(PluginBundleManager.BUNDLE_EXTRA_PLAYING_POSITION);

            AudioManager amgr = (AudioManager) context.getSystemService(Context.AUDIO_SERVICE);
            boolean playing = amgr.isMusicActive();

            if (playingPos == EditActivity.DOESNT_MATTER ||
                (playingPos == EditActivity.YES && playing) ||
                (playingPos == EditActivity.NO && !playing)) {
                String button = bundle.getString(PluginBundleManager.BUNDLE_EXTRA_STRING_BUTTON);
                int keycode = EditActivity.BUTTONS.get(button);

                Log.d(EditActivity.TAG,
                      "Broadcasting ACTION_MEDIA_BUTTON with ACTION_DOWN and then ACTION_UP with " +
                      button + " (" + keycode + ")");

                Intent buttonIntent = new Intent(Intent.ACTION_MEDIA_BUTTON);
                buttonIntent.putExtra(Intent.EXTRA_KEY_EVENT,
                                      new KeyEvent(KeyEvent.ACTION_DOWN, keycode));
                context.sendOrderedBroadcast(buttonIntent, null);

                buttonIntent.putExtra(Intent.EXTRA_KEY_EVENT,
                                      new KeyEvent(KeyEvent.ACTION_UP, keycode));
                context.sendOrderedBroadcast(buttonIntent, null);
            }
        }
    }
}
